--------------------------------------------------------------------------------
local db_function = {}
local db_locale   = {title = {"dialogue-box.title",""}}
local mod_gui = require("mod-gui")
db_variable = {dialogues = nil, game_loaded = false}
global.trigger = "db_triggers_off"

--------------------------------------------------------------------------------
-- Global loading --
function db_function.require_load(event)
  db_variable.game_loaded = false
end
event_handler.addListener("on_load",db_function.require_load,true)
event_handler.addListener("on_configuration_changed",db_function.require_load,true)
event_handler.addListener(defines.events.on_player_joined_game,db_function.require_load)

function db_function.load_setup(event)
  if not db_variable.game_loaded then
    -- Check if the interface exists
    local check_interface, _ = pcall(db_function.check_interface)
    if not check_interface then
      remote.add_interface("dialogue-box", {get_dialogues = function(key) return {} end, set_dialogues = function(key, value) end})
      db_variable.error_report_interface = "No interfaces has been loaded. This means that no mods has dialogue-box integration set up."
      db_function.print_errors(event)
    end
    -- Load dialogues
    db_variable.dialogues = remote.call("dialogue-box", "get_dialogues", "dialogues") or 0
    -- Check if dialogues has been properly loaded
    local check_dialogues, _ = pcall(db_function.check_dialogues)
    if check_dialogues then
      db_variable.game_loaded = true
      if not game.is_multiplayer() or global.trigger == "db_triggers_on" then
        db_function.update_box()
      end
    end
  end
end
event_handler.addListener(defines.events.on_tick, db_function.load_setup)

--------------------------------------------------------------------------------
-- Player loading --
function toggle_triggers(event)
  local status, style = pcall(db_function.check_gui_unyfier, event)
  for index, luaPlayer in pairs(game.players) do
    local window = game.players[index].gui.top
    if status then
      window = mod_gui.get_button_flow(game.players[index])
    else
      style = nil
    end
    if window["db_toggle_triggers"] then window["db_toggle_triggers"].destroy() end
    db_variable.toggle_triggers = window.add{type = "sprite-button", sprite = global.trigger, name = "db_toggle_triggers", style=style}
  end
  if global.trigger == "db_triggers_on" then
    db_function.update_box()
  else
    event_handler.removeListener(nil, nil,"dialogue")
  end
end

--------------------------------------------------------------------------------
-- Error handling --
function db_function.check_dialogues()
  for _,data in pairs(db_variable.dialogues) do
    return nil
  end
end
function db_function.check_interface()  return remote.interfaces["dialogue-box"]["get_dialogues"] end
function db_function.check_index(event) return event.player_index end
function db_function.check_gui_unyfier(event)
  return settings.get_player_settings(game.players[event.player_index])["gu_button_style_setting"].value or "slot_button_notext"
end


function db_function.print_errors(event)
  local is_critical = false
  if db_variable.error_report_interface then
    game.print(db_variable.error_report_interface, {255,0,0})
    is_critical = true
  end
  if is_critical then
    game.print("The game will probably crash if the mod is not set up properly. Please disable it or contact developers.",{255,0,0})
  end
end

--------------------------------------------------------------------------------
-- Update Dialogue Box --
function db_function.update_box()
  for _,data in pairs(db_variable.dialogues) do
    function dialogue_trigger(event)
      if data.active and remote.call("dialogue-box",data.condition,event) then
        -- Disable repetitions
        data.active = false
        event_handler.removeListener(data.trigger, dialogue_trigger)
        -- Save dialogues to the global
        remote.call("dialogue-box", "set_dialogues", "dialogues", db_variable.dialogues)
        -- Set title to "speaking with person"
        db_locale.title[2] = data.person

        for index,luaPlayer in pairs(game.players) do
          if luaPlayer.connected then
            local screen_element = luaPlayer.gui.screen
            db_function.box_graphics(screen_element, _, data)
          end
        end
      end
    end
    if data.active then event_handler.addListener(data.trigger, dialogue_trigger, false, "dialogue") end
  end
end

--------------------------------------------------------------------------------
-- Graphics --
function db_function.box_graphics(screen_element, _, data)
  db_destroy_gui(screen_element)

  local frame_main = screen_element.add{type="frame", name="main".._, direction="vertical"}
  db_add_titlebar(frame_main, db_locale.title, "db_close_button")
  local padding_correction = frame_main.add{type="flow", name="correction".._, style = "dialogue_correction", direction="vertical"}
  local frame_dialogue = padding_correction.add{type="frame", name="dialogue".._, style="dialogue_inner_frame",direction="horizontal"}
  local sprite_image = frame_dialogue.add{type="sprite-button", name=data.person, sprite=data.person, style = "dialogue_image_sprite"}
  local label_text = frame_dialogue.add{type="label", name="text".._, caption=data.caption}

  frame_main.style.size = {600, 200}
  frame_dialogue.style.size = {588, 155}
  label_text.style.size = {455,145}

  frame_main.auto_center = true
  label_text.style.single_line = false
  label_text.style.margin = 5
end

--------------------------------------------------------------------------------
return nil
