--------------------------------------------------------------------------------
local db_function = {}
local db_locale = {title = {"dialogue-log.title"}}
local mod_gui = require("mod-gui")
db_variable.log_open = nil

--------------------------------------------------------------------------------
-- Global loading --

--------------------------------------------------------------------------------
-- Player Loading --
function db_function.check_gui_unyfier(event)
  return settings.get_player_settings(game.players[event.player_index])["gu_button_style_setting"].value or "slot_button_notext"
end
function on_player_created(event)
  local status, style = pcall(db_function.check_gui_unyfier, event)
  local window = game.players[event.player_index].gui.top
  if status then
    window = mod_gui.get_button_flow(game.players[event.player_index])
  else
    style = nil
  end

  if window["db_menu_button"] then window["db_menu_button"].destroy() end
  if window["db_toggle_triggers"] then window["db_toggle_triggers"].destroy() end

  db_variable.menu_button = window.add{type = "sprite-button", sprite = "menu-icon", name = "db_menu_button", style=style}

  if game.is_multiplayer() then
    db_variable.toggle_triggers = window.add{type = "sprite-button", sprite = global.trigger, name = "db_toggle_triggers", style=style}
  end
end
event_handler.addListener(defines.events.on_player_created, on_player_created)
event_handler.addListener(defines.events.on_runtime_mod_setting_changed, on_player_created)

--------------------------------------------------------------------------------
-- GUI handling --
function db_function.on_gui_click(event)
  if event.element and event.element.valid then
    if event.element.name == "db_menu_button" then
      db_variable.log_open = nil
      local window = game.players[event.player_index].gui.screen["frame-main"]
      if window then window.destroy()
      else
        db_variable.main_frame = db_function.open_window(event.player_index)
        db_function.update_window(event)
      end
    elseif event.element.name == "db_close_button" then
      db_destroy_gui(game.players[event.player_index].gui.screen)
    elseif event.element.name == "db_toggle_triggers" then
      if global.trigger == "db_triggers_on" then
        global.trigger = "db_triggers_off"
      else
        global.trigger = "db_triggers_on"
      end
      toggle_triggers(event)
    else
      for _,data in pairs(db_variable.dialogues) do
        if event.element.name == data.person then
          db_variable.log_open = data.person
          if not game.players[event.player_index].gui.screen["frame-main"] then
            db_function.open_window(event.player_index)
          end
          db_function.update_window(event)
          break
        end
      end
    end
  end
end
event_handler.addListener(defines.events.on_gui_click, db_function.on_gui_click)

--------------------------------------------------------------------------------
-- Open window --
function db_function.close_dialogue(event)
  local window = game.players[event.player_index].gui.screen["frame-main"]
  if window then window.destroy() end
end
event_handler.addListener("close-dialogue", db_function.close_dialogue)

--------------------------------------------------------------------------------
-- Open window --
function db_function.open_window(player_index)
  db_destroy_gui(game.players[player_index].gui.screen)
  local main_frame = game.players[player_index].gui.screen.add{type="frame", name="frame-main", direction="vertical"}
  main_frame.auto_center = true
  main_frame.style.size = {1066, 600}
end

--------------------------------------------------------------------------------
-- Update window --
function db_function.update_window(event)
  local main_frame = game.players[event.player_index].gui.screen["frame-main"]
  db_destroy_gui(main_frame)
  db_add_titlebar(main_frame, db_locale.title, "db_close_button", true)

  local main_flow = main_frame.add{type="flow", name="flow-main", direction="horizontal"}
  local peoplelist_frame = main_flow.add{type="frame", direction="vertical", name="frame-list", style="dialogue_inner_frame"}
  peoplelist_frame.style.size = {200, 548}

  for _,data in pairs(db_variable.dialogues) do
    local state = true
    for p,pers in pairs(peoplelist_frame.children) do
      if pers then
        if pers.name == data.person then state = false end
      end
    end
    if not data.active and state then
      peoplelist_frame.add{type="button", name=data.person, caption=data.person, style = "dialogue_person_button"}
    end
  end

  if db_variable.log_open then
    local property_frame = main_flow.add{type="frame", name="property_flow", direction="vertical", style="dialogue_inner_frame"}
    local person_title = property_frame.add{type="label", style="frame_title", caption=db_variable.log_open}
    local person_sprite = property_frame.add{type="sprite-button", name="", sprite=db_variable.log_open, style = "dialogue_image_sprite"}
    local dialogue_flow = main_flow.add{type="scroll-pane", name="person_sprite"} --, direction = "vertical"

    property_frame.style.size = {200, 548}
    person_sprite.style.size = {180,233}
    dialogue_flow.style.size = {616, 548}
    dialogue_flow.horizontal_scroll_policy = "never"
    dialogue_flow.vertical_scroll_policy = "auto-and-reserve-space"
    property_frame.style.left_margin = 9
    property_frame.style.left_padding = 9
    person_title.style.top_padding = 2
    property_frame.style.left_margin = 9
    dialogue_flow.style.left_margin = 9

    for _,data in pairs(db_variable.dialogues) do
      if data.person == db_variable.log_open and not data.active then
        local dialogue_frame = dialogue_flow.add{type="frame",style="dialogue_inner_frame"}
        local label_text = dialogue_frame.add{type="label", caption=data.caption}

        dialogue_frame.style.width = 606
        dialogue_frame.style.left_padding = 9
        dialogue_frame.style.bottom_padding = 2
        label_text.style.single_line = false
      end
    end
  else
    local main_page = main_flow.add{type="button",name="Main_page",style="Main_page"}
    main_page.style.left_margin = 9
  end
end

--------------------------------------------------------------------------------
return nil
